require 'spec_helper'

describe NdSync::Remote do
  NdSync.remote_site = 'http://localhost:3210'

  class RemoteModel < ActiveResource::Base
    include NdSync::Remote

    def to_batch json
      json.(self, :name)
    end
  end

  it "loads site from NdSync" do
    expect(RemoteModel.site).to eq URI('http://localhost:3210')
  end

  it "transmits objects requesting by sync_pending" do
    stub_request(:get, "localhost:3210/remote_models.json").with(query: {sync_pending: 'true'}).
      to_return(body: [{id: 5}].to_json, :status => 200)

    webs = RemoteModel.to_transmit
    expect(webs.size    ).to eq 1
    expect(webs.first.id).to eq 5
  end

  it "generates objects in JSON to be uploaded" do
    rm = RemoteModel.new(name: 'test')

    expect(RemoteModel.to_batch([rm])).to eq({remote_models: [{name: 'test'}]}.to_json)
  end

  describe "transmitted_to" do
    it "updates remote object with local ID" do
      req = stub_request(:put, "localhost:3210/remote_models/3/synched/5.json")

      RemoteModel.new(id: 3).transmitted_to double(id: 5)

      expect(req).to have_been_requested
    end
  end
end
