require 'spec_helper'

describe NdSync do
  NdSync.remote_site = 'http://localhost:3210'

  class Local < ActiveRecord::Base
    include NdSync::Local

    transcriber do
      copy_fields :id => :web_id
    end

    def origin_step_field
      :step
    end

    def origin_table
      self.class.table_name
    end

    def origin_id_field
      self.class.primary_key
    end

    alias :origin_id :id
  end

  class Remote < ActiveResource::Base
    include NdSync::Remote

    def to_batch(json)
      json.(self, :id)
    end

    transcriber do
      copy_fields :web_id => :id
    end
  end

  before(:all) do
    Local.connection.create_table :locals do |t|
      t.integer :web_id
      t.integer :step
      t.integer :desired_step
    end
  end

  before do
    Local.delete_all
  end

  it "allows to download data" do
    stub_request(:get, "localhost:3210/remotes.json").with(query: {sync_pending: 'true'}).
      to_return(body: [{id: 5}].to_json, :status => 200)
    req = stub_request(:put, /localhost:3210\/remotes\/5\/synched\/\d\.json/)

    expect do
      NdTransmission::Base.new(Remote => Local).transmit
    end.to change(Local, :count).by(1)
    expect(req).to have_been_requested
  end

  it "allows to upload data" do
    Local.create!(web_id: 5, step: 1, desired_step: 2)
    req = stub_request(:post, "localhost:3210/remotes/batch.json").
      with(body: {remotes: [{id: 5}]}).
      to_return(body: [{id: 5}].to_json, :status => 200)

    expect do
      NdTransmission::Base.new(Local => Remote).transmit
    end.to change(Local.pending, :count).by(-1)
    expect(req).to have_been_requested
  end
end
